@@ -2,7 +2,7 @@
 #
 # Workbook - A class for writing the Excel XLSX Workbook file.
 #
-# Copyright 2013-2016, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2017, John McNamara, jmcnamara@cpan.org
 #
 
 # Standard packages.
@@ -10,6 +10,7 @@ import sys
 import re
 import os
 import operator
+import warnings
 from warnings import warn
 from datetime import datetime
 from zipfile import ZipFile, ZIP_DEFLATED
@@ -48,12 +49,16 @@ class Workbook(xmlwriter.XMLwriter):
     # Public API.
     #
     ###########################################################################
+    chartsheet_class = Chartsheet
+    worksheet_class = Worksheet
 
-    def __init__(self, filename=None, options={}):
+    def __init__(self, filename=None, options=None):
         """
         Constructor.
 
         """
+        if options is None:
+            options = {}
 
         super(Workbook, self).__init__()
 
@@ -66,7 +71,7 @@ class Workbook(xmlwriter.XMLwriter):
         self.strings_to_urls = options.get('strings_to_urls', True)
         self.nan_inf_to_errors = options.get('nan_inf_to_errors', False)
         self.default_date_format = options.get('default_date_format', None)
-        self.optimization = options.get('constant_memory', False)
+        self.constant_memory = options.get('constant_memory', False)
         self.in_memory = options.get('in_memory', False)
         self.excel2003_style = options.get('excel2003_style', False)
         self.remove_timezone = options.get('remove_timezone', False)
@@ -123,7 +128,7 @@ class Workbook(xmlwriter.XMLwriter):
 
         # We can't do 'constant_memory' mode while doing 'in_memory' mode.
         if self.in_memory:
-            self.optimization = False
+            self.constant_memory = False
 
         # Add the default cell format.
         if self.excel2003_style:
@@ -157,7 +162,7 @@ class Workbook(xmlwriter.XMLwriter):
         """Close workbook when exiting "with" statement."""
         self.close()
 
-    def add_worksheet(self, name=None):
+    def add_worksheet(self, name=None, worksheet_class=None):
         """
         Add a new worksheet to the Excel workbook.
 
@@ -168,9 +173,12 @@ class Workbook(xmlwriter.XMLwriter):
             Reference to a worksheet object.
 
         """
-        return self._add_sheet(name, is_chartsheet=False)
+        if worksheet_class is None:
+            worksheet_class = self.worksheet_class
 
-    def add_chartsheet(self, name=None):
+        return self._add_sheet(name, worksheet_class=worksheet_class)
+
+    def add_chartsheet(self, name=None, chartsheet_class=None):
         """
         Add a new chartsheet to the Excel workbook.
 
@@ -181,9 +189,12 @@ class Workbook(xmlwriter.XMLwriter):
             Reference to a chartsheet object.
 
         """
-        return self._add_sheet(name, is_chartsheet=True)
+        if chartsheet_class is None:
+            chartsheet_class = self.chartsheet_class
 
-    def add_format(self, properties={}):
+        return self._add_sheet(name, worksheet_class=chartsheet_class)
+
+    def add_format(self, properties=None):
         """
         Add a new Format to the Excel Workbook.
 
@@ -200,7 +211,8 @@ class Workbook(xmlwriter.XMLwriter):
             format_properties = {'font_name': 'Arial', 'font_size': 10,
                                  'theme': 1 * -1}
 
-        format_properties.update(properties)
+        if properties:
+            format_properties.update(properties)
 
         xf_format = Format(format_properties,
                            self.xf_format_indices,
@@ -615,7 +627,6 @@ class Workbook(xmlwriter.XMLwriter):
         # Add XML sub-files to the Zip file with their Excel filename.
         for os_filename, xml_filename, is_binary in xml_files:
             if self.in_memory:
-                # The files are in-memory StringIOs.
                 if is_binary:
                     xlsx_file.writestr(xml_filename, os_filename.getvalue())
                 else:
@@ -628,11 +639,30 @@ class Workbook(xmlwriter.XMLwriter):
 
         xlsx_file.close()
 
-    def _add_sheet(self, name, is_chartsheet):
+    def _add_sheet(self, name, is_chartsheet=None, worksheet_class=None):
         # Utility for shared code in add_worksheet() and add_chartsheet().
 
+        if is_chartsheet is not None:
+            warnings.warn(
+                "'is_chartsheet' has been deprecated and "
+                "may be removed in a future version. Use 'worksheet_class' "
+                "to get the same result",
+                PendingDeprecationWarning)
+
+        if is_chartsheet is None and worksheet_class is None:
+            raise ValueError(
+                "You must provide 'is_chartsheet' or 'worksheet_class'")
+
+        if worksheet_class:
+            worksheet = worksheet_class()
+        else:
+            if is_chartsheet:
+                worksheet = self.chartsheet_class()
+            else:
+                worksheet = self.worksheet_class()
+
         sheet_index = len(self.worksheets_objs)
-        name = self._check_sheetname(name, is_chartsheet)
+        name = self._check_sheetname(name, isinstance(worksheet, Chartsheet))
 
         # Initialization data to pass to the worksheet.
         init_data = {
@@ -640,7 +670,7 @@ class Workbook(xmlwriter.XMLwriter):
             'index': sheet_index,
             'str_table': self.str_table,
             'worksheet_meta': self.worksheet_meta,
-            'optimization': self.optimization,
+            'constant_memory': self.constant_memory,
             'tmpdir': self.tmpdir,
             'date_1904': self.date_1904,
             'strings_to_numbers': self.strings_to_numbers,
@@ -653,11 +683,6 @@ class Workbook(xmlwriter.XMLwriter):
             'remove_timezone': self.remove_timezone,
         }
 
-        if is_chartsheet:
-            worksheet = Chartsheet()
-        else:
-            worksheet = Worksheet()
-
         worksheet._initialize(init_data)
 
         self.worksheets_objs.append(worksheet)
@@ -683,6 +708,10 @@ class Workbook(xmlwriter.XMLwriter):
             else:
                 sheetname = self.sheet_name + str(self.sheetname_count)
 
+        # Check if sheetname is empty.
+        if sheetname == '':
+            raise Exception("Excel worksheet name cannot be empty")
+
         # Check that sheet sheetname is <= 31. Excel limit.
         if len(sheetname) > 31:
             raise Exception("Excel worksheet name '%s' must be <= 31 chars." %
